package p20_konto;

public class Konto {
	private final int numer;
	private int saldo;
	private Osoba wlasciciel;
	
	public Konto(int numer, int saldo, Osoba wlasciciel) {
		this.numer = numer;
		this.saldo = saldo;
		this.wlasciciel = wlasciciel;
	}

	@Override
	public String toString() {
		return "Konto nr " + numer + ", saldo " + saldo + ", wł. " + wlasciciel;
	}

	public Osoba getWlasciciel() {
		return wlasciciel;
	}

	public void setWlasciciel(Osoba wlasciciel) {
		this.wlasciciel = wlasciciel;
	}

	public int getNumer() {
		return numer;
	}

	public int getSaldo() {
		return saldo;
	}
	
	// Dla pola numer nie tworzę settera, gdyż numer konta się nie zmienia.
	// Dodatkowo jest to zapewnione modyfikatorem final.
	
	// Dla pola saldo nie tworzę settera, gdyż salda nie ustawia się "bez żadnego trybu",
	// tylko saldo można zmodyfikować poprzez operacje biznesowe: wplata, wyplata, przelew.
	
	public void wplata(int kwota) {
		if(kwota <= 0) {
			throw new IllegalArgumentException("Kwota nie jest dodatnia");
		}
		saldo += kwota;
	}
	
	public void wyplata(int kwota) throws BrakSrodkow {
		if(kwota <= 0) {
			throw new IllegalArgumentException("Kwota nie jest dodatnia");
		}
		if(kwota > saldo) {
			throw new BrakSrodkow("Za mało kasy podczas wypłaty");
		}
		saldo -= kwota;
	}
	
	public void przelew(int kwota, Konto kontoDocelowe) throws BrakSrodkow {
		if(kwota <= 0) {
			throw new IllegalArgumentException("Kwota nie jest dodatnia");
		}
		if(kwota > saldo) {
			throw new BrakSrodkow("Za mało kasy podczas przelewu");
		}
		this.saldo -= kwota;
		kontoDocelowe.saldo += kwota;
	}
}
